/*
	
	LightMeter.cpp:
	Access the ambient light sensor of a MacBook Pro via SuperCollider.
	
    Copyright (c) 2007 Daniel Jones. All rights reserved.
	http://www.erase.net/
	
	----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/


#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <mach/mach.h>

#include "SC_PlugIn.h"
#include "lmucommon.h"

// Period in seconds between sampling data -- this is a CPU-intensive
// operation, and so should not be done once per block. In any case, the
// LMU only appears to update a few times per second.
#define METER_PERIOD 0.05

static InterfaceTable *ft;

struct LightMeter : public Unit
{
	io_connect_t port;
	int value_left,
		value_right;
	int sample_counter;
};

extern "C"
{
	void load(InterfaceTable *table);
	void LightMeter_next(LightMeter *unit, int samples);
	void LightMeter_Ctor(LightMeter* unit);
	void LightMeter_Dtor(LightMeter* unit);
};


// Init UGen
void LightMeter_Ctor(LightMeter *unit)
{
	kern_return_t kr;
	io_service_t serviceObject;
	io_service_t servicePort;
  
	SETCALC(LightMeter_next);
	
	// NOTE: May work for some PowerBook G4 models with service name
	//       changed to "IOI2CDeviceLMU" - UNTESTED.
	serviceObject = IOServiceGetMatchingService(kIOMasterPortDefault,
                    IOServiceMatching("AppleLMUController"));

	if (!serviceObject)
	{
		printf("LightMeter: Could not initialise LMU hardware.\n");
		printf("LightMeter: A MacBook or MacBook Pro is required to run this patch.\n");
		return;
	}

	kr = IOServiceOpen(serviceObject, mach_task_self(), 0, &servicePort);
	IOObjectRelease(serviceObject);
	if (kr != KERN_SUCCESS)
	{
		printf("LightMeter: LMU service could not be started.\n");
		return;
	}

	unit->port = servicePort;
	unit->value_left = 0;
	unit->value_right = 0;
	unit->sample_counter = 0;
	
	LightMeter_next(unit, 1);
}

void LightMeter_Dtor(LightMeter *unit)
{
}

void LightMeter_next(LightMeter *unit, int samples)
{	
	if (--unit->sample_counter < 0)
	{
		kern_return_t kr;
		io_connect_t port = unit->port;

		SInt32 left = 0, right = 0;
		IOItemCount scalarInputCount = 0,
					scalarOutputCount = 2;
		
		kr = IOConnectMethodScalarIScalarO(port, kGetSensorReadingID,
			scalarInputCount, scalarOutputCount, &left, &right);

		if (kr != KERN_SUCCESS)
		{
			printf("LightMeter: LMU retrieval failed.\n");
			return;
		}
		
		unit->value_left = left;
		unit->value_right = right;
		unit->sample_counter = METER_PERIOD *
			unit->mWorld->mSampleRate /
			unit->mWorld->mFullRate.mBufLength;
	}
	
	ZOUT0(0) = unit->value_left;
	ZOUT0(1) = unit->value_right;
}

void load(InterfaceTable *table)
{
	ft = table;
	DefineDtorUnit(LightMeter);
}
